@@ -1,12 +1,13 @@
 #!/usr/bin/env python3
 import logging
-import re
+
+# import re
 
 log = logging.getLogger("helpers")
 
 
 def get_kwargs(kwargs, key, default=None):
-    if not key in kwargs or not kwargs[key]:
+    if key not in kwargs or not kwargs[key]:
         return default
     return kwargs[key]
 
@@ -29,3 +30,23 @@ def key_wanted(key, filter=None, excl_filter=None):
     else:
         log.debug(f"key_wanted: key {key} does not match filter {filter} so key excluded")
         return False
+
+
+def get_resp_defn(key, defns):
+    """
+    look for a definition for the supplied key
+    """
+    # print(key, defns)
+    if not key:
+        return None
+    try:
+        key = key.decode("utf-8")
+    except UnicodeDecodeError:
+        log.info(f"key decode error for {key}")
+    for defn in defns:
+        if key == defn[0]:
+            # print(key, defn)
+            return defn
+    # did not find definition for this key
+    log.info(f"No defn found for {key} key")
+    return [key, key, "", ""]
