@@ -1,6 +1,6 @@
 import logging
 
-from .protocol import AbstractProtocol
+from .abstractprotocol import AbstractProtocol
 from .protocol_helpers import crcPI as crc
 
 log = logging.getLogger("pi30")
@@ -132,17 +132,29 @@ COMMANDS = {
         ],
         "regex": "PCVV(\\d\\d\\.\\d)$",
     },
-    "PEPD": {
-        "name": "PEPD",
-        "description": "Set the enabled / disabled state of various Inverter settings (e.g. buzzer, overload, interrupt alarm)",
-        "help": " -- examples: PEABJ/PDKUVXYZ (enable A buzzer, B overload bypass, J power saving / disable K LCD go to default after 1min, U overload restart, V overtemp restart, X backlight, Y alarm on primary source interrupt, Z fault code record)",
+    "PE": {
+        "name": "PE",
+        "description": "Set the enabled state of an Inverter setting",
+        "help": " -- examples: PEa - enable a (buzzer) [a=buzzer, b=overload bypass, j=power saving, K=LCD go to default after 1min, u=overload restart, v=overtemp restart, x=backlight, y=alarm on primary source interrupt, z=fault code record]",
         "type": "SETTER",
         "response": [["ack", "Command execution", {"NAK": "Failed", "ACK": "Successful"}]],
         "test_responses": [
             b"(NAK\x73\x73\r",
             b"(ACK\x39\x20\r",
         ],
-        "regex": "PE(.*)/PD(.*)$",
+        "regex": "PE(.+)$",
+    },
+    "PD": {
+        "name": "PD",
+        "description": "Set the disabled state of an Inverter setting",
+        "help": " -- examples: PDa - disable a (buzzer) [a=buzzer, b=overload bypass, j=power saving, K=LCD go to default after 1min, u=overload restart, v=overtemp restart, x=backlight, y=alarm on primary source interrupt, z=fault code record]",
+        "type": "SETTER",
+        "response": [["ack", "Command execution", {"NAK": "Failed", "ACK": "Successful"}]],
+        "test_responses": [
+            b"(NAK\x73\x73\r",
+            b"(ACK\x39\x20\r",
+        ],
+        "regex": "PD(.+)$",
     },
     "PF": {
         "name": "PF",
@@ -154,7 +166,6 @@ COMMANDS = {
             b"(NAK\x73\x73\r",
             b"(ACK\x39\x20\r",
         ],
-        "regex": "",
     },
     "PGR": {
         "name": "PGR",
@@ -274,7 +285,6 @@ COMMANDS = {
         "test_responses": [
             b"(00000 00000 01 01 00 059 045 053 068 00 00 000 0040 0580 0000 50.00 13\x39\xB9\r",
         ],
-        "regex": "",
     },
     "QBOOT": {
         "name": "QBOOT",
@@ -284,7 +294,6 @@ COMMANDS = {
         "test_responses": [
             "",
         ],
-        "regex": "",
     },
     "QDI": {
         "name": "QDI",
@@ -337,6 +346,7 @@ COMMANDS = {
                     "Phase 3 of 3 Phase output",
                     "Phase 1 of 2 phase output",
                     "Phase 2 of 2 phase output",
+                    "unknown output phase",
                 ],
             ],
             ["float", "Battery Redischarge Voltage", "V"],
@@ -360,7 +370,6 @@ COMMANDS = {
         "test_responses": [
             b"(230.0 50.0 0030 42.0 54.0 56.4 46.0 60 0 0 2 0 0 0 0 0 1 1 0 0 1 0 54.0 0 1 000\x9E\x60\r",
         ],
-        "regex": "",
     },
     "QFLAG": {
         "name": "QFLAG",
@@ -390,7 +399,6 @@ COMMANDS = {
         "test_responses": [
             b"(EakxyDbjuvz\x2F\x29\r",
         ],
-        "regex": "",
     },
     "QID": {
         "name": "QID",
@@ -401,7 +409,6 @@ COMMANDS = {
         "test_responses": [
             b"(9293333010501\xBB\x07\r",
         ],
-        "regex": "",
     },
     "QMCHGCR": {
         "name": "QMCHGCR",
@@ -412,7 +419,6 @@ COMMANDS = {
         "test_responses": [
             b"",
         ],
-        "regex": "",
     },
     "QMOD": {
         "name": "QMOD",
@@ -436,7 +442,6 @@ COMMANDS = {
         "test_responses": [
             b"(S\x64\x39\r",
         ],
-        "regex": "",
     },
     "QMUCHGCR": {
         "name": "QMUCHGCR",
@@ -447,7 +452,6 @@ COMMANDS = {
         "test_responses": [
             b"",
         ],
-        "regex": "",
     },
     "QOPM": {
         "name": "QOPM",
@@ -466,13 +470,13 @@ COMMANDS = {
                     "Phase 3 of 3 Phase output",
                     "Phase 1 of 2 phase output",
                     "Phase 2 of 2 phase output",
+                    "unknown output phase",
                 ],
             ]
         ],
         "test_responses": [
             b"",
         ],
-        "regex": "",
     },
     "QPGS": {
         "name": "QPGS",
@@ -569,6 +573,7 @@ COMMANDS = {
                     "Phase 3 of 3 phase output",
                     "Phase 1 of 2 phase output",
                     "Phase 2 of 2 phase output",
+                    "Unknown Output Mode",
                 ],
             ],
             [
@@ -581,11 +586,15 @@ COMMANDS = {
             ["int", "Max AC charger current", "A"],
             ["int", "PV input current", "A"],
             ["int", "Battery discharge current", "A"],
+            ["float", "Unknown float", ""],
+            ["string", "Unknown flags?", ""],
         ],
         "test_responses": [
             b"(1 92931701100510 B 00 000.0 00.00 230.6 50.00 0275 0141 005 51.4 001 100 083.3 002 00574 00312 003 10100110 1 2 060 120 10 04 000\xcc#\r",
+            b"(1 92912102100033 B 00 000.0 00.00 120.1 59.99 0048 0000 000 53.1 000 059 000.0 000 00154 00016 000 00000110 7 1 060 120 030 00 000 000.0 00\xe7c\r",
+            b"QPGS0?\xda\r",
         ],
-        "regex": "QPGS(\\d)$",
+        "regex": "QPGS(\\d+)$",
     },
     "QPI": {
         "name": "QPI",
@@ -598,7 +607,6 @@ COMMANDS = {
         "test_responses": [
             b"(PI30\x9a\x0b\r",
         ],
-        "regex": "",
     },
     "QPIGS": {
         "name": "QPIGS",
@@ -648,13 +656,13 @@ COMMANDS = {
         "test_responses": [
             b"(000.0 00.0 230.0 49.9 0161 0119 003 460 57.50 012 100 0069 0014 103.8 57.45 00000 00110110 00 00 00856 010\x24\x8c\r",
         ],
-        "regex": "",
     },
     "QPIRI": {
         "name": "QPIRI",
         "description": "Current Settings inquiry",
         "help": " -- queries the current settings from the Inverter",
         "type": "QUERY",
+        "response_type": "SEQUENTIAL",
         "response": [
             ["float", "AC Input Voltage", "V"],
             ["float", "AC Input Current", "A"],
@@ -668,7 +676,21 @@ COMMANDS = {
             ["float", "Battery Under Voltage", "V"],
             ["float", "Battery Bulk Charge Voltage", "V"],
             ["float", "Battery Float Charge Voltage", "V"],
-            ["option", "Battery Type", ["AGM", "Flooded", "User"]],
+            [
+                "option",
+                "Battery Type",
+                [
+                    "AGM",
+                    "Flooded",
+                    "User",
+                    "TBD",
+                    "Pylontech",
+                    "WECO",
+                    "Soltaro",
+                    "LIb-protocol compatible",
+                    "3rd party Lithium",
+                ],
+            ],
             ["int", "Max AC Charging Current", "A"],
             ["int", "Max Charging Current", "A"],
             ["option", "Input Voltage Range", ["Appliance", "UPS"]],
@@ -689,7 +711,7 @@ COMMANDS = {
             ],
             ["int", "Max Parallel Units", "units"],
             [
-                "keyed",
+                "str_keyed",
                 "Machine Type",
                 {"00": "Grid tie", "01": "Off Grid", "10": "Hybrid"},
             ],
@@ -705,6 +727,7 @@ COMMANDS = {
                     "Phase 3 of 3 Phase output",
                     "Phase 1 of 2 phase output",
                     "Phase 2 of 2 phase output",
+                    "unknown output",
                 ],
             ],
             ["float", "Battery Redischarge Voltage", "V"],
@@ -728,8 +751,10 @@ COMMANDS = {
         "test_responses": [
             b"(230.0 21.7 230.0 50.0 21.7 5000 4000 48.0 46.0 42.0 56.4 54.0 0 10 010 1 0 0 6 01 0 0 54.0 0 1\x6F\x7E\r",
             b"(120.0 25.0 120.0 60.0 25.0 3000 3000 48.0 46.0 44.0 58.4 54.4 2 30 060 1 2 0 9 01 0 6 54.0 0 1 000 0\x8f\xed\r",
+            b"(230.0 13.0 230.0 50.0 13.0 3000 2400 24.0 23.0 21.0 28.2 27.0 0 30 50 0 2 1 - 01 1 0 26.0 0 0\xb9\xbd\r",
+            b"(230.0 21.7 230.0 50.0 21.7 5000 5000 48.0 47.0 46.5 57.6 57.6 5 30 080 0 1 2 1 01 0 0 52.0 0 1\x03$\r",
+            b"(230.0 21.7 230.0 50.0 21.7 5000 5000 48.0 47.0 46.5 57.6 57.6 9 30 080 0 1 2 1 01 0 0 52.0 0 1\x9c\x6f\r",
         ],
-        "regex": "",
     },
     "QPIWS": {
         "name": "QPIWS",
@@ -739,9 +764,9 @@ COMMANDS = {
         "response": [
             [
                 "stat_flags",
-                "Warning status",
+                "Warning",
                 [
-                    "Reserved",
+                    "",
                     "Inverter fault",
                     "Bus over fault",
                     "Bus under fault",
@@ -771,15 +796,14 @@ COMMANDS = {
                     "MPPT overload fault",
                     "MPPT overload warning",
                     "Battery too low to charge warning",
-                    "Reserved",
-                    "Reserved",
+                    "",
+                    "",
                 ],
             ]
         ],
         "test_responses": [
-            b"(00000100000000000000000000000000\xFE\x82\r",
+            b"(00000100000000001000000000000000\x56\xA6\r",
         ],
-        "regex": "",
     },
     "QVFW": {
         "name": "QVFW",
@@ -790,7 +814,6 @@ COMMANDS = {
         "test_responses": [
             b"(VERFW:00072.70\x53\xA7\r",
         ],
-        "regex": "",
     },
     "QVFW2": {
         "name": "QVFW2",
@@ -801,7 +824,6 @@ COMMANDS = {
         "test_responses": [
             b"",
         ],
-        "regex": "",
     },
 }
 
@@ -835,5 +857,5 @@ class pi30(AbstractProtocol):
             crc_high, crc_low = crc(response[:-3])
             if response[-3:-1] != bytes([crc_high, crc_low]):
                 return False, {"ERROR": ["Invalid response CRC", ""]}
-        log.debug("check_response_valid: CRCs match")
+        log.debug("CRCs match")
         return True, {}
